#  Copyright (C) 2012   Christian Heimes (christian@python.org)
#  Licensed to PSF under a Contributor Agreement.
#

__all__ = ("sha3_224", "sha3_256", "sha3_384", "sha3_512")
from _sha3 import sha3_224, sha3_256, sha3_384, sha3_512

# monkey patch _hashlib
import hashlib as _hashlib

_hashlib_constructor = _hashlib.__get_builtin_constructor

def __get_builtin_constructor(name):
    if name in set(['sha3_224', 'sha3_256', 'sha3_384', 'sha3_512',
                   'SHA3_224', 'SHA3_256', 'SHA3_384', 'SHA3_512']):
        bs = name[5:]
        if bs == '224':
            return sha3_224
        elif bs == '256':
            return sha3_256
        elif bs == '384':
            return sha3_384
        elif bs == '512':
            return sha3_512
    return _hashlib_constructor(name)

if not hasattr(_hashlib, "sha3_512"):
    _hashlib.sha3_224 = sha3_224
    _hashlib.sha3_256 = sha3_256
    _hashlib.sha3_384 = sha3_384
    _hashlib.sha3_512 = sha3_512
    _hashlib.__get_builtin_constructor = __get_builtin_constructor



#!/usr/bin/env python
# Copyright (C) 2012   Christian Heimes (christian@python.org)
# Licensed to PSF under a Contributor Agreement.
#
# cleanup Keccak sources

import os
import re

CPP1 = re.compile("^//(.*)")
CPP2 = re.compile("\ //(.*)")

STATICS = ("void ", "int ", "HashReturn ", "const UINT64 ", "UINT16 ")

HERE = os.path.dirname(os.path.abspath(__file__))
KECCAK = os.path.join(HERE, "keccak")

def getfiles():
    for name in os.listdir(KECCAK):
        name = os.path.join(KECCAK, name)
        if os.path.isfile(name):
            yield name

def cleanup(f):
    buf = []
    for line in f:
        # mark all functions and global data as static
        if line.startswith(STATICS):
            buf.append("static " + line)
            continue
        # remove UINT64 typedef, we have our own
        if line.startswith("typedef unsigned long long int"):
            buf.append("/* %s */\n" % line.strip())
            continue
        # remove #include "brg_endian.h"
        if "brg_endian.h" in line:
            buf.append("/* %s */\n" % line.strip())
            continue
        # transform C++ comments into ANSI C comments
        line = CPP1.sub(r"/* \1 */", line)
        line = CPP2.sub(r" /* \1 */", line)
        buf.append(line)
    return "".join(buf)

for name in getfiles():
    with open(name) as f:
        res = cleanup(f)
    with open(name, "w") as f:
        f.write(res)



#!/usr/bin/env python
import sys
import os
import subprocess
from glob import glob
from distutils.core import setup, Command
from distutils.extension import Extension


class TestCommand(Command):
    """Hack for setup.py with implicit build_ext -i
    """
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        # hack inplace build into build_ext
        options = self.distribution.command_options
        extoptions = options.setdefault("build_ext", {})
        extoptions["inplace"] = ("hack", True)

    def remove_ext(self):
        """Remove extensions

        All Python 2.x versions share the same library name. Remove the
        file to fix version mismatch errors.
        """
        root = os.getcwd()
        for fname in os.listdir(root):
            if fname.endswith(("so", "dylib", "pyd")):
                os.unlink(os.path.join(root, fname))

    def run(self):
        if sys.version_info[0] < 3:
            self.remove_ext()

        # force a build with build_ext -i
        self.run_command("build")

        errno = subprocess.check_call([sys.executable, "tests.py"])
        raise SystemExit(errno)


exts = []
sha3_depends =  []
sha3_depends.extend(glob("Modules/_sha3/keccak/*.c"))
sha3_depends.extend(glob("Modules/_sha3/keccak/*.h"))
sha3_depends.extend(glob("Modules/_sha3/keccak/*.macros"))
exts.append(Extension("_sha3", ["Modules/_sha3/sha3module.c"],
                      depends=sha3_depends))

long_description = []
with open("README.txt") as f:
    long_description.append(f.read())
with open("CHANGES.txt") as f:
    long_description.append(f.read())

setup(
    name="pysha3",
    version="0.2.1",
    ext_modules=exts,
    py_modules=["sha3"],
    cmdclass = {"test": TestCommand},
    author="Christian Heimes",
    author_email="christian@python.org",
    maintainer="Christian Heimes",
    maintainer_email="christian@python.org",
    url="https://bitbucket.org/tiran/pykeccak",
    keywords="sha3 sha-3 keccak hash",
    license="PSFL (Keccak: CC0 1.0 Universal)",
    description="SHA-3 (Keccak) for Python 2.6 - 3.4",
    long_description="\n".join(long_description),
    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Python Software Foundation License',
        'Natural Language :: English',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: C',
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        #"Programming Language :: Python :: 3.4",
        'Topic :: Security :: Cryptography',
    ),
)
